"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.actionsFor = void 0;

var R = _interopRequireWildcard(require("ramda"));

var RA = _interopRequireWildcard(require("ramda-adjunct"));

var _array = require("./array");

var _object = require("./object");

var _boolean = require("./boolean");

var _number = require("./number");

var _any = require("./any");

var _forString = require("./string/forString");

var _custom = require("../custom");

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj["default"] = obj; return newObj; } }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var actionsFor = function actionsFor(stateShape, customReducers) {
  var paths = recursivelyGeneratePaths(stateShape);
  var actions = {
    create: createFor(stateShape, customReducers)
  };
  paths.forEach(function (path) {
    var isPathToBranch = isBranch(R.path(path, stateShape));

    if (isPathToBranch) {
      actions = addActionsToBranch(actions, path, stateShape, customReducers);
    } else {
      actions = addActionsToLeaf(actions, path, stateShape, customReducers);
    }
  });
  return actions;
};

exports.actionsFor = actionsFor;

var actionsForLeafOrBranch = function actionsForLeafOrBranch(leafOrBranch) {
  var pathToLeafOrBranch = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
  var stateShape = arguments.length > 2 ? arguments[2] : undefined;
  var customReducers = arguments.length > 3 ? arguments[3] : undefined;
  leafOrBranch.create = createFor(stateShape, customReducers, pathToLeafOrBranch);
  return leafOrBranch;
};

var addActionsToBranch = function addActionsToBranch(actions, path, stateShape, customReducers) {
  var branch = R.path(path, stateShape);
  return R.assocPath(path, actionsForLeafOrBranch(branch, path, stateShape, customReducers), actions);
};

var addActionsToLeaf = function addActionsToLeaf(actions, path, stateShape, customReducers) {
  return R.assocPath(path, actionsForLeafOrBranch({}, path, stateShape, customReducers), actions);
};

var createFor = function createFor(stateShape, customReducers) {
  var pathToLeafOrBranch = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];
  var initialState = pathToLeafOrBranch.length >= 1 ? R.path(pathToLeafOrBranch, stateShape) : stateShape;
  var actionCreators;
  var basicActionCreators = (0, _any.forAny)(pathToLeafOrBranch);
  var asArray = (0, _array.forArray)(pathToLeafOrBranch);
  var asBoolean = (0, _boolean.forBoolean)(pathToLeafOrBranch);
  var asNumber = (0, _number.forNumber)(pathToLeafOrBranch);
  var asObject = (0, _object.forObject)(pathToLeafOrBranch);
  var asString = (0, _forString.forString)(pathToLeafOrBranch);
  var custom = (0, _custom.makeCustomActions)(customReducers, pathToLeafOrBranch);

  if (RA.isBoolean(initialState)) {
    actionCreators = asBoolean;
  } else if (RA.isArray(initialState)) {
    actionCreators = asArray;
  } else if (RA.isNumber(initialState)) {
    actionCreators = asNumber;
  } else if (RA.isPlainObject(initialState)) {
    actionCreators = asObject;
  } else if (RA.isString(initialState)) {
    actionCreators = asString;
  }

  return _objectSpread({}, basicActionCreators, actionCreators, {
    asArray: asArray,
    asBoolean: asBoolean,
    asNumber: asNumber,
    asObject: asObject,
    asString: asString,
    custom: custom
  }, custom);
};

var isBranch = function isBranch(leafOrBranch) {
  return !Array.isArray(leafOrBranch) && _typeof(leafOrBranch) === "object" && Object.values(leafOrBranch).length >= 1;
};

var recursivelyGeneratePaths = function recursivelyGeneratePaths(stateShape) {
  var paths = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
  var currentPath = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];

  if (RA.isPlainObject(stateShape)) {
    Object.entries(stateShape).forEach(function (_ref) {
      var _ref2 = _slicedToArray(_ref, 2),
          key = _ref2[0],
          val = _ref2[1];

      var newPath = [].concat(_toConsumableArray(currentPath), [key]);
      paths.push(newPath);
      recursivelyGeneratePaths(val, paths, newPath);
    });
  }

  return paths;
};