"use strict";

var _lodash = _interopRequireDefault(require("lodash"));

var _redux = require("redux");

var _2 = _interopRequireDefault(require("../.."));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

describe("leaf.create.apply(callback): returns an action that, when dispatched, updates the leaf's state to the return value of callback(state, entireState)", function () {
  describe("Documentation example 1", function () {
    describe("GIVEN setup of initialState and store as documented", function () {
      var initialState = {
        bool: false,
        num: 2,
        str: 'foo',
        arr: [1, 2, 3],
        obj: {}
      };

      var _reduxLeaves = (0, _2.default)(initialState),
          _reduxLeaves2 = _slicedToArray(_reduxLeaves, 2),
          reducer = _reduxLeaves2[0],
          actions = _reduxLeaves2[1];

      var store;
      beforeEach(function () {
        store = (0, _redux.createStore)(reducer);
      });
      describe("WHEN we execute store.dispatch(actions.str.create.apply(state => state.toUpperCase()))", function () {
        beforeEach(function () {
          store.dispatch(actions.str.create.apply(function (state) {
            return state.toUpperCase();
          }));
        });
        test("THEN the store's state.str is 'FOO'", function () {
          expect(store.getState().str).toBe('FOO');
        });
        describe("AND we execute store.dispatch(actions.create.apply(state => ({ num: state.num, arr: state.arr })))", function () {
          beforeEach(function () {
            store.dispatch(actions.create.apply(function (state) {
              return {
                num: state.num,
                arr: state.arr
              };
            }));
          });
          test("THEN the store's state is { num: 2, arr: [1, 2, 3] }", function () {
            expect(store.getState()).toEqual({
              num: 2,
              arr: [1, 2, 3]
            });
          });
          describe("AND we execute store.dispatch(actions.arr.create.apply((leafState, entireState) => (leafState.map(element => element * entireState.num))))))", function () {
            beforeEach(function () {
              store.dispatch(actions.arr.create.apply(function (leafState, entireState) {
                return leafState.map(function (element) {
                  return element * entireState.num;
                });
              }));
            });
            test("THEN the store's state is { num: 2, arr: [2, 4, 6] }", function () {
              expect(store.getState()).toEqual({
                num: 2,
                arr: [2, 4, 6]
              });
            });
          });
        });
      });
    });
  });
  describe("GIVEN non-trivially nested API (as in the documentation)", function () {
    var initialState = {
      counter: 1,
      foo: ["bar"],
      nested: {
        deep: {},
        state: {
          manageable: "maybe...?"
        }
      }
    };
    describe("WHEN [reducer, actions] = reduxLeaves(initialState)", function () {
      var _reduxLeaves3 = (0, _2.default)(initialState),
          _reduxLeaves4 = _slicedToArray(_reduxLeaves3, 2),
          reducer = _reduxLeaves4[0],
          actions = _reduxLeaves4[1];

      test("THEN actions.counter.create.apply is a function", function () {
        expect(_typeof(actions.counter.create.apply)).toBe("function");
      });
      test("AND actions.foo.create.apply is a function", function () {
        expect(_typeof(actions.foo.create.apply)).toBe("function");
      });
      test("AND actions.nested.deep.create.apply is a function", function () {
        expect(_typeof(actions.nested.deep.create.apply)).toBe("function");
      });
      test("AND actions.nested.state.manageable.create.apply is a function", function () {
        expect(_typeof(actions.nested.state.manageable.create.apply)).toBe("function");
      });
      describe("AND store = createStore(reducer)", function () {
        var store;
        beforeEach(function () {
          store = (0, _redux.createStore)(reducer);
        });
        test("THEN store is initialised with state = initialState", function () {
          expect(store.getState()).toEqual(initialState);
        });
        describe("AND we dispatch actions.counter.create.apply(n => n * 7)", function () {
          beforeEach(function () {
            store.dispatch(actions.counter.create.apply(function (n) {
              return n * 7;
            }));
          });
          test("THEN actions.counter updates to 7", function () {
            var state = store.getState();
            expect(state.counter).toBe(7);
            expect(state).toEqual(_objectSpread({}, initialState, {
              counter: 7
            }));
          });
        });
        describe("AND we dispatch actions.foo.create.apply(arr => ['foo', ...arr])", function () {
          beforeEach(function () {
            store.dispatch(actions.foo.create.apply(function (arr) {
              return ['foo'].concat(_toConsumableArray(arr));
            }));
          });
          test("THEN actions.foo updates to ['foo', 'bar']", function () {
            var state = store.getState();
            expect(state.foo).toEqual(['foo', 'bar']);
            expect(state).toEqual(_objectSpread({}, initialState, {
              foo: ['foo', 'bar']
            }));
          });
        });
        describe("AND we dispatch actions.nested.deep.create.apply(obj => ({ ...obj, arbitrarily: true }))", function () {
          beforeEach(function () {
            store.dispatch(actions.nested.deep.create.apply(function (obj) {
              return _objectSpread({}, obj, {
                arbitrarily: true
              });
            }));
          });
          test("THEN actions.nested.deep updates to { arbitrarily: true }", function () {
            var state = store.getState();
            expect(state.nested.deep).toEqual({
              arbitrarily: true
            });
            expect(state).toEqual(_objectSpread({}, initialState, {
              nested: _objectSpread({}, initialState.nested, {
                deep: {
                  arbitrarily: true
                }
              })
            }));
          });
        });
        describe("AND we dispatch actions.nested.state.manageable.create.apply(str => str.concat(' DEFINITELY!'))", function () {
          beforeEach(function () {
            store.dispatch(actions.nested.state.manageable.create.apply(function (str) {
              return str.concat(' DEFINITELY!');
            }));
          });
          test("THEN actions.nested.state.manageable updates to 'maybe...? DEFINITELY!'", function () {
            var state = store.getState();
            expect(state.nested.state.manageable).toEqual('maybe...? DEFINITELY!');
            expect(state).toEqual(_objectSpread({}, initialState, {
              nested: _objectSpread({}, initialState.nested, {
                state: _objectSpread({}, initialState.nested.state, {
                  manageable: 'maybe...? DEFINITELY!'
                })
              })
            }));
          });
        });
      });
    });
  });
});