"use strict";

var _lodash = _interopRequireDefault(require("lodash"));

var _redux = require("redux");

var _2 = _interopRequireDefault(require("../../.."));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

describe("leaf.create.concat(array): returns an action that, when dispatched, updates the leaf's state by non-mutatively concatenating it with array", function () {
  describe("GIVEN initialState is an object", function () {
    var initialState = {
      empty: [],
      integers: [1, 2, 3]
    };
    describe("WHEN [reducer, actions] = reduxLeaves(initialState)", function () {
      var _reduxLeaves = (0, _2.default)(initialState),
          _reduxLeaves2 = _slicedToArray(_reduxLeaves, 2),
          reducer = _reduxLeaves2[0],
          actions = _reduxLeaves2[1];

      test("THEN actions.empty.create.concat is a function", function () {
        expect(_typeof(actions.empty.create.concat)).toBe("function");
      });
      test("AND actions.integers.create.concat is a function", function () {
        expect(_typeof(actions.integers.create.concat)).toBe("function");
      });
      describe("AND store = createStore(reducer)", function () {
        var store;
        beforeEach(function () {
          store = (0, _redux.createStore)(reducer);
        });
        test("THEN store is initialised with state = initialState", function () {
          expect(store.getState()).toEqual(initialState);
        });
        describe("AND we dispatch actions.empty.create.concat([1, 2, 3])", function () {
          beforeEach(function () {
            store.dispatch(actions.empty.create.concat([1, 2, 3]));
          });
          test("THEN actions.empty state updates non-mutatively to [1, 2, 3]", function () {
            var state = store.getState();
            expect(state).toEqual(_objectSpread({}, initialState, {
              empty: [1, 2, 3]
            }));
            expect(initialState.empty).toEqual([]);
          });
        });
        describe("AND we dispatch actions.integers.create.concat([4, 5, 6])", function () {
          beforeEach(function () {
            store.dispatch(actions.integers.create.concat([4, 5, 6]));
          });
          test("THEN actions.integers state updates non-mutatively to [1, 2, 3,4, 5, 6]", function () {
            var state = store.getState();
            expect(state).toEqual(_objectSpread({}, initialState, {
              integers: [1, 2, 3, 4, 5, 6]
            }));
            expect(initialState.integers).toEqual([1, 2, 3]);
          });
        });
      });
    });
  });
});
describe("leaf.create.concat(...strings): returns an action that, when dispatched, updates the leaf's state by non-mutatively concatenating it with strings", function () {
  describe("GIVEN initialState is an object", function () {
    var initialState = {
      foo: "foo"
    };
    describe("WHEN [reducer, actions] = reduxLeaves(initialState)", function () {
      var _reduxLeaves3 = (0, _2.default)(initialState),
          _reduxLeaves4 = _slicedToArray(_reduxLeaves3, 2),
          reducer = _reduxLeaves4[0],
          actions = _reduxLeaves4[1];

      test("THEN actions.foo.create.concat is a function", function () {
        expect(_typeof(actions.foo.create.concat)).toBe("function");
      });
      describe("AND store = createStore(reducer)", function () {
        var store;
        beforeEach(function () {
          store = (0, _redux.createStore)(reducer);
        });
        test("THEN store is initialised with state = initialState", function () {
          expect(store.getState()).toEqual(initialState);
        });
        describe("AND we dispatch actions.foo.create.concat('bar')", function () {
          beforeEach(function () {
            store.dispatch(actions.foo.create.concat('bar'));
          });
          test("THEN actions.foo state updates non-mutatively to 'foobar'", function () {
            var state = store.getState();
            expect(state).toEqual(_objectSpread({}, initialState, {
              foo: 'foobar'
            }));
            expect(initialState.foo).toEqual('foo');
          });
        });
        describe("AND we dispatch actions.foo.create.concat('bar', '!')", function () {
          beforeEach(function () {
            store.dispatch(actions.foo.create.concat('bar', '!'));
          });
          test("THEN actions.foo state updates non-mutatively to 'foobar!'", function () {
            var state = store.getState();
            expect(state).toEqual(_objectSpread({}, initialState, {
              foo: 'foobar!'
            }));
            expect(initialState.foo).toEqual('foo');
          });
        });
      });
    });
  });
});