"use strict";

var _lodash = _interopRequireDefault(require("lodash"));

var _redux = require("redux");

var _2 = _interopRequireDefault(require("../.."));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

describe("leaf.create.clear(toNull = false): returns an action that, when dispatched, clear's the leaf's state", function () {
  describe("GIVEN non-trivially nested API (as in the documentation)", function () {
    var initialState = {
      counter: 1,
      foo: ["bar"],
      nested: {
        deep: {},
        state: {
          manageable: "maybe...?"
        }
      }
    };
    describe("WHEN [reducer, actions] = reduxLeaves(initialState)", function () {
      var _reduxLeaves = (0, _2.default)(initialState),
          _reduxLeaves2 = _slicedToArray(_reduxLeaves, 2),
          reducer = _reduxLeaves2[0],
          actions = _reduxLeaves2[1];

      test("THEN actions.counter.create.clear is a function", function () {
        expect(_typeof(actions.counter.create.clear)).toBe("function");
      });
      test("AND actions.foo.create.clear is a function", function () {
        expect(_typeof(actions.foo.create.clear)).toBe("function");
      });
      test("AND actions.nested.deep.create.clear is a function", function () {
        expect(_typeof(actions.nested.deep.create.clear)).toBe("function");
      });
      test("AND actions.nested.state.manageable.create.clear is a function", function () {
        expect(_typeof(actions.nested.state.manageable.create.clear)).toBe("function");
      });
      describe("AND store = createStore(reducer, otherState)", function () {
        var store;
        var otherState = {
          counter: 5,
          foo: ["FOOBAR"],
          nested: {
            deep: {
              props: true
            },
            state: {
              manageable: "let's find out"
            }
          }
        };
        beforeEach(function () {
          store = (0, _redux.createStore)(reducer, otherState);
        });
        test("THEN store is initialised with state = otherState", function () {
          expect(store.getState()).toEqual(otherState);
        });
        describe("AND we dispatch actions.counter.create.clear()", function () {
          beforeEach(function () {
            store.dispatch(actions.counter.create.clear());
          });
          test("THEN actions.counter updates to 0", function () {
            var state = store.getState();
            expect(state.counter).toBe(0);
            expect(state).toEqual(_objectSpread({}, otherState, {
              counter: 0
            }));
          });
        });
        describe("AND we dispatch actions.counter.create.clear(true)", function () {
          beforeEach(function () {
            store.dispatch(actions.counter.create.clear(true));
          });
          test("THEN actions.counter updates to null", function () {
            var state = store.getState();
            expect(state.counter).toBeNull();
            expect(state).toEqual(_objectSpread({}, otherState, {
              counter: null
            }));
          });
        });
        describe("AND we dispatch actions.foo.create.clear()", function () {
          beforeEach(function () {
            store.dispatch(actions.foo.create.clear());
          });
          test("THEN actions.foo updates to []", function () {
            var state = store.getState();
            expect(state.foo).toEqual([]);
            expect(state).toEqual(_objectSpread({}, otherState, {
              foo: []
            }));
          });
        });
        describe("AND we dispatch actions.foo.create.clear(true)", function () {
          beforeEach(function () {
            store.dispatch(actions.foo.create.clear(true));
          });
          test("THEN actions.foo updates to null", function () {
            var state = store.getState();
            expect(state.foo).toBeNull();
            expect(state).toEqual(_objectSpread({}, otherState, {
              foo: null
            }));
          });
        });
        describe("AND we dispatch actions.nested.deep.create.clear()", function () {
          beforeEach(function () {
            store.dispatch(actions.nested.deep.create.clear());
          });
          test("THEN actions.nested.deep updates to {}", function () {
            var state = store.getState();
            expect(state.nested.deep).toEqual({});
            expect(state).toEqual(_objectSpread({}, otherState, {
              nested: _objectSpread({}, otherState.nested, {
                deep: {}
              })
            }));
          });
        });
        describe("AND we dispatch actions.nested.deep.create.clear(true)", function () {
          beforeEach(function () {
            store.dispatch(actions.nested.deep.create.clear(true));
          });
          test("THEN actions.nested.deep updates to null", function () {
            var state = store.getState();
            expect(state.nested.deep).toBeNull();
            expect(state).toEqual(_objectSpread({}, otherState, {
              nested: _objectSpread({}, otherState.nested, {
                deep: null
              })
            }));
          });
        });
        describe("AND we dispatch actions.nested.state.manageable.create.clear()", function () {
          beforeEach(function () {
            store.dispatch(actions.nested.state.manageable.create.clear());
          });
          test("THEN actions.nested.state.manageable updates to ''", function () {
            var state = store.getState();
            expect(state.nested.state.manageable).toBe('');
            expect(state).toEqual(_objectSpread({}, otherState, {
              nested: _objectSpread({}, otherState.nested, {
                state: _objectSpread({}, otherState.nested.state, {
                  manageable: ''
                })
              })
            }));
          });
        });
        describe("AND we dispatch actions.nested.state.manageable.create.clear(true)", function () {
          beforeEach(function () {
            store.dispatch(actions.nested.state.manageable.create.clear(true));
          });
          test("THEN actions.nested.state.manageable updates to null", function () {
            var state = store.getState();
            expect(state.nested.state.manageable).toBeNull();
            expect(state).toEqual(_objectSpread({}, otherState, {
              nested: _objectSpread({}, otherState.nested, {
                state: _objectSpread({}, otherState.nested.state, {
                  manageable: null
                })
              })
            }));
          });
        });
        describe("AND we dispatch actions.create.clear(true)", function () {
          beforeEach(function () {
            store.dispatch(actions.create.clear(true));
          });
          test("THEN state updates to null", function () {
            var state = store.getState();
            expect(state).toBeNull();
          });
        });
        describe("AND we dispatch actions.create.clear()", function () {
          beforeEach(function () {
            store.dispatch(actions.create.clear());
          });
          test("THEN state updates to {}", function () {
            var state = store.getState();
            expect(state).toEqual({});
          });
        });
      });
    });
  });
});