"use strict";

var _lodash = _interopRequireDefault(require("lodash"));

var _redux = require("redux");

var _2 = _interopRequireDefault(require("../.."));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

describe("leaf.create.reset(): returns an action that, when dispatched, updates the leaf's state to the reducer's initialised state", function () {
  describe("Documentation example 1", function () {
    describe("GIVEN setup of initialState, otherState and store as documented", function () {
      var initialState = {
        num: 2,
        arr: [1, 2, 3]
      };
      var otherState = {
        num: 11,
        arr: ['a', 'b', 'c']
      };

      var _reduxLeaves = (0, _2.default)(initialState),
          _reduxLeaves2 = _slicedToArray(_reduxLeaves, 2),
          reducer = _reduxLeaves2[0],
          actions = _reduxLeaves2[1];

      var store;
      beforeEach(function () {
        store = (0, _redux.createStore)(reducer, otherState);
      });
      test("THEN store is initialised with otherState", function () {
        expect(store.getState()).toEqual(otherState);
      });
      describe("WHEN we execute store.dispatch(actions.num.create.reset()))", function () {
        beforeEach(function () {
          store.dispatch(actions.num.create.reset());
        });
        test("THEN the store's state.num is 2", function () {
          expect(store.getState().num).toBe(2);
        });
        describe("AND we execute store.dispatch(actions.create.reset())", function () {
          beforeEach(function () {
            store.dispatch(actions.create.reset());
          });
          test("THEN the store's state is { num: 2, arr: [1, 2, 3] }", function () {
            expect(store.getState()).toEqual({
              num: 2,
              arr: [1, 2, 3]
            });
          });
        });
      });
    });
  });
  describe("GIVEN non-trivially nested API (as in the documentation)", function () {
    var initialState = {
      counter: 1,
      foo: ["bar"],
      nested: {
        deep: {},
        state: {
          manageable: "maybe...?"
        }
      }
    };
    describe("WHEN [reducer, actions] = reduxLeaves(initialState)", function () {
      var _reduxLeaves3 = (0, _2.default)(initialState),
          _reduxLeaves4 = _slicedToArray(_reduxLeaves3, 2),
          reducer = _reduxLeaves4[0],
          actions = _reduxLeaves4[1];

      test("THEN actions.counter.create.reset is a function", function () {
        expect(_typeof(actions.counter.create.reset)).toBe("function");
      });
      test("AND actions.foo.create.reset is a function", function () {
        expect(_typeof(actions.foo.create.reset)).toBe("function");
      });
      test("AND actions.nested.deep.create.reset is a function", function () {
        expect(_typeof(actions.nested.deep.create.reset)).toBe("function");
      });
      test("AND actions.nested.state.manageable.create.reset is a function", function () {
        expect(_typeof(actions.nested.state.manageable.create.reset)).toBe("function");
      });
      describe("AND store = createStore(reducer, otherState)", function () {
        var store;
        var otherState = {
          counter: 5,
          foo: ["FOOBAR"],
          nested: {
            deep: {
              props: true
            },
            state: {
              manageable: "let's find out"
            }
          }
        };
        beforeEach(function () {
          store = (0, _redux.createStore)(reducer, otherState);
        });
        test("THEN store is initialised with state = otherState", function () {
          expect(store.getState()).toEqual(otherState);
        });
        describe("AND we dispatch actions.counter.create.reset()", function () {
          beforeEach(function () {
            store.dispatch(actions.counter.create.reset());
          });
          test("THEN actions.counter resets to 1", function () {
            var state = store.getState();
            expect(state.counter).toBe(1);
            expect(state).toEqual(_objectSpread({}, otherState, {
              counter: 1
            }));
          });
        });
        describe("AND we dispatch actions.foo.create.reset()", function () {
          beforeEach(function () {
            store.dispatch(actions.foo.create.reset());
          });
          test("THEN actions.foo resets to ['bar']", function () {
            var state = store.getState();
            expect(state.foo).toEqual(['bar']);
            expect(state).toEqual(_objectSpread({}, otherState, {
              foo: ['bar']
            }));
          });
        });
        describe("AND we dispatch actions.nested.deep.create.reset()", function () {
          beforeEach(function () {
            store.dispatch(actions.nested.deep.create.reset());
          });
          test("THEN actions.nested.deep resets to {}", function () {
            var state = store.getState();
            expect(state.nested.deep).toEqual({});
            expect(state).toEqual(_objectSpread({}, otherState, {
              nested: _objectSpread({}, otherState.nested, {
                deep: {}
              })
            }));
          });
        });
        describe("AND we dispatch actions.nested.state.manageable.create.reset()", function () {
          beforeEach(function () {
            store.dispatch(actions.nested.state.manageable.create.reset());
          });
          test("THEN actions.nested.state.manageable resets to 'maybe...?'", function () {
            var state = store.getState();
            expect(state.nested.state.manageable).toEqual('maybe...?');
            expect(state).toEqual(_objectSpread({}, otherState, {
              nested: _objectSpread({}, otherState.nested, {
                state: _objectSpread({}, otherState.nested.state, {
                  manageable: 'maybe...?'
                })
              })
            }));
          });
        });
      });
    });
  });
});