"use strict";

var _lodash = _interopRequireDefault(require("lodash"));

var _redux = require("redux");

var _2 = _interopRequireDefault(require("../.."));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

describe("leaf.create.update(value): returns an action that, when dispatched, updates the leaf's state to value", function () {
  describe("Documentation example 1", function () {
    describe("GIVEN setup of initialState and store as documented", function () {
      var initialState = {
        bool: false,
        num: 2,
        str: 'foo',
        arr: [1, 2, 3]
      };

      var _reduxLeaves = (0, _2.default)(initialState),
          _reduxLeaves2 = _slicedToArray(_reduxLeaves, 2),
          reducer = _reduxLeaves2[0],
          actions = _reduxLeaves2[1];

      var store;
      beforeEach(function () {
        store = (0, _redux.createStore)(reducer);
      });
      describe("WHEN we execute store.dispatch(actions.str.create.update('I can put anything here'))", function () {
        beforeEach(function () {
          store.dispatch(actions.str.create.update('I can put anything here'));
        });
        test("THEN the store's state.str is 'I can put anything here'", function () {
          expect(store.getState().str).toBe('I can put anything here');
        });
        describe("AND we execute store.dispatch(actions.create.update({ any: { properties: true } }))", function () {
          beforeEach(function () {
            store.dispatch(actions.create.update({
              any: {
                properties: true
              }
            }));
          });
          test("THEN the store's state is { any: { properties: { true }} }", function () {
            expect(store.getState()).toEqual({
              any: {
                properties: true
              }
            });
          });
        });
      });
    });
  });
  describe("GIVEN non-trivially nested API (as in the documentation)", function () {
    var initialState = {
      counter: 1,
      foo: ["bar"],
      nested: {
        deep: {},
        state: {
          manageable: "maybe...?"
        }
      }
    };
    describe("WHEN [reducer, actions] = reduxLeaves(initialState)", function () {
      var _reduxLeaves3 = (0, _2.default)(initialState),
          _reduxLeaves4 = _slicedToArray(_reduxLeaves3, 2),
          reducer = _reduxLeaves4[0],
          actions = _reduxLeaves4[1];

      test("THEN actions.counter.create.update is a function", function () {
        expect(_typeof(actions.counter.create.update)).toBe("function");
      });
      test("AND actions.foo.create.update is a function", function () {
        expect(_typeof(actions.foo.create.update)).toBe("function");
      });
      test("AND actions.nested.deep.create.update is a function", function () {
        expect(_typeof(actions.nested.deep.create.update)).toBe("function");
      });
      test("AND actions.nested.state.manageable.create.update is a function", function () {
        expect(_typeof(actions.nested.state.manageable.create.update)).toBe("function");
      });
      describe("AND store = createStore(reducer)", function () {
        var store;
        beforeEach(function () {
          store = (0, _redux.createStore)(reducer);
        });
        test("THEN store is initialised with state = initialState", function () {
          expect(store.getState()).toEqual(initialState);
        });
        describe("AND we dispatch actions.counter.create.update(9)", function () {
          beforeEach(function () {
            store.dispatch(actions.counter.create.update(9));
          });
          test("THEN actions.counter state updates non-mutatively to 9", function () {
            var state = store.getState();
            expect(state).toEqual(_objectSpread({}, initialState, {
              counter: 9
            }));
            expect(initialState.counter).toBe(1);
          });
        });
        describe("AND we dispatch actions.foo.create.update(['f', 'o', 'o'])", function () {
          beforeEach(function () {
            store.dispatch(actions.foo.create.update(['f', 'o', 'o']));
          });
          test("THEN actions.foo state updates non-mutatively to ['f', 'o', 'o']", function () {
            var state = store.getState();
            expect(state).toEqual(_objectSpread({}, initialState, {
              foo: ['f', 'o', 'o']
            }));
            expect(initialState.foo).toEqual(['bar']);
          });
        });
        describe("AND we dispatch actions.nested.deep.create.update({ here: true })", function () {
          beforeEach(function () {
            store.dispatch(actions.nested.deep.create.update({
              here: true
            }));
          });
          test("THEN actions.nested.deep updates to { here: true }", function () {
            var state = store.getState();
            expect(state).toEqual(_objectSpread({}, initialState, {
              nested: _objectSpread({}, initialState.nested, {
                deep: {
                  here: true
                }
              })
            }));
            expect(initialState.nested.deep).toEqual({});
          });
        });
        describe("AND we dispatch actions.nested.state.manageabl.createe.update('thanks to redux-leaves!')", function () {
          beforeEach(function () {
            store.dispatch(actions.nested.state.manageable.create.update('thanks to redux-leaves!'));
          });
          test("THEN actions.nested.state.manageable updates to 'thanks to redux-leaves!'", function () {
            var state = store.getState();
            expect(state).toEqual(_objectSpread({}, initialState, {
              nested: _objectSpread({}, initialState.nested, {
                state: _objectSpread({}, initialState.nested.state, {
                  manageable: 'thanks to redux-leaves!'
                })
              })
            }));
            expect(initialState.nested.state.manageable).toBe("maybe...?");
          });
        });
      });
    });
  });
});