"use strict";

var _lodash = _interopRequireDefault(require("lodash"));

var _redux = require("redux");

var _reduxLeaves7 = require("../reduxLeaves");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

describe("API: reduxLeaves(initialState, [customLogic = {}])", function () {
  describe("Example 1: custom action creator with no arguments", function () {
    describe("GIVEN initialState and customLogic", function () {
      var initialState = {
        foo: 3,
        bar: 4
      };
      var customLogic = {
        square: function square(leafState) {
          return Math.pow(leafState, 2);
        }
      };
      describe("WHEN we pass initialState and customLogic to reduxLeaves", function () {
        var _reduxLeaves = (0, _reduxLeaves7.reduxLeaves)(initialState, customLogic),
            _reduxLeaves2 = _slicedToArray(_reduxLeaves, 2),
            reducer = _reduxLeaves2[0],
            actions = _reduxLeaves2[1];

        var store;
        beforeEach(function () {
          return store = (0, _redux.createStore)(reducer);
        });
        test("THEN store initialises with initialState", function () {
          expect(store.getState()).toEqual(initialState);
        });
        test("AND create.custom.square is defined for actions.foo and actions.bar", function () {
          expect(_typeof(actions.foo.create.custom.square)).toBe("function");
          expect(_typeof(actions.bar.create.custom.square)).toBe("function");
        });
        describe("AND we dispatch actions.foo.create.custom.square()", function () {
          beforeEach(function () {
            store.dispatch(actions.foo.create.custom.square());
          });
          test("THEN foo has squared, but not bar", function () {
            expect(store.getState()).toEqual({
              foo: 9,
              bar: 4
            });
          });
          describe("AND we dispatch actions.bar.create.custom.square()", function () {
            beforeEach(function () {
              store.dispatch(actions.bar.create.custom.square());
            });
            test("THEN bar has also squared now", function () {
              expect(store.getState()).toEqual({
                foo: 9,
                bar: 16
              });
            });
          });
        });
      });
    });
  });
  describe("Example 2: custom action creator using payload and wholeState", function () {
    describe("GIVEN initialState and customLogic", function () {
      var initialState = {
        foo: 2,
        bar: [2, 4, 6, 8, 10]
      };
      var customLogic = {
        exponentiate: function exponentiate(leafState, _ref) {
          var payload = _ref.payload;
          return Math.pow(leafState, payload);
        },
        remove: function remove(leafState, _ref2, wholeState) {
          var payload = _ref2.payload;
          return leafState.filter(function (e) {
            return e != wholeState[payload];
          });
        }
      };
      describe("WHEN we pass initialState and customLogic to reduxLeaves", function () {
        var _reduxLeaves3 = (0, _reduxLeaves7.reduxLeaves)(initialState, customLogic),
            _reduxLeaves4 = _slicedToArray(_reduxLeaves3, 2),
            reducer = _reduxLeaves4[0],
            actions = _reduxLeaves4[1];

        var store;
        beforeEach(function () {
          return store = (0, _redux.createStore)(reducer);
        });
        test("THEN store initialises with initialState", function () {
          expect(store.getState()).toEqual(initialState);
        });
        test("AND create.custom.exponentiate is defined for actions.foo", function () {
          expect(_typeof(actions.foo.create.custom.exponentiate)).toBe("function");
        });
        test("AND create.custom is defined for actions.bar", function () {
          expect(_typeof(actions.bar.create.custom.remove)).toBe("function");
        });
        describe("AND we pass arguments to custom.exponentiate before dispatching", function () {
          var expWithOneArg = actions.foo.create.custom.exponentiate(2);
          var expWithTwoArgs = actions.foo.create.custom.exponentiate(3, 4);
          beforeEach(function () {
            store.dispatch(expWithTwoArgs);
          });
          test("THEN custom.exponentiate sets payload to be the first argument", function () {
            expect(expWithOneArg.payload).toBe(2);
            expect(expWithTwoArgs.payload).toBe(3);
          });
          test("AND the store state updates as expected", function () {
            expect(store.getState()).toEqual(_objectSpread({}, initialState, {
              foo: 8
            }));
          });
          describe("AND we pass an argument to custom.remove before dispatching", function () {
            var removeUsingFoo = actions.bar.create.custom.remove("foo");
            beforeEach(function () {
              store.dispatch(removeUsingFoo);
            });
            test("THEN custom.remove sets payload to be the first argument", function () {
              expect(removeUsingFoo.payload).toEqual("foo");
            });
            test("AND the store state updates as expected", function () {
              expect(store.getState()).toEqual({
                foo: 8,
                bar: [2, 4, 6, 10]
              });
            });
          });
        });
      });
    });
  });
  describe("Example 3: more detailed customisation with argsToPayload", function () {
    describe("GIVEN initialState and customLogic", function () {
      var initialState = {
        foo: [2, 4, 6, 8, 10]
      };
      var customLogic = {
        remove: {
          argsToPayload: function argsToPayload() {
            for (var _len = arguments.length, values = new Array(_len), _key = 0; _key < _len; _key++) {
              values[_key] = arguments[_key];
            }

            return values;
          },
          reducer: function reducer(leafState, _ref3) {
            var payload = _ref3.payload;
            return leafState.filter(function (e) {
              return !payload.includes(e);
            });
          }
        }
      };
      describe("WHEN we pass initialState and customLogic to reduxLeaves", function () {
        var _reduxLeaves5 = (0, _reduxLeaves7.reduxLeaves)(initialState, customLogic),
            _reduxLeaves6 = _slicedToArray(_reduxLeaves5, 2),
            reducer = _reduxLeaves6[0],
            actions = _reduxLeaves6[1];

        var store;
        beforeEach(function () {
          return store = (0, _redux.createStore)(reducer);
        });
        test("THEN store initialises with initialState", function () {
          expect(store.getState()).toEqual(initialState);
        });
        test("AND create.custom.remove is defined for actions.foo", function () {
          expect(_typeof(actions.foo.create.custom.remove)).toBe("function");
        });
        describe("AND we pass arguments to custom.remove before dispatching", function () {
          var removeWithOneArg = actions.foo.create.custom.remove(4);
          var removeWithTwoArgs = actions.foo.create.custom.remove(4, 8);
          beforeEach(function () {
            store.dispatch(removeWithTwoArgs);
          });
          test("THEN custom.remove sets payload to be the first argument", function () {
            expect(removeWithOneArg.payload).toEqual([4]);
            expect(removeWithTwoArgs.payload).toEqual([4, 8]);
          });
          test("AND the store state updates as expected", function () {
            expect(store.getState().foo).toEqual([2, 6, 10]);
          });
        });
      });
    });
  });
});