"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.leafReducer = void 0;

var R = _interopRequireWildcard(require("ramda"));

var RA = _interopRequireWildcard(require("ramda-adjunct"));

var _immer = _interopRequireDefault(require("immer"));

var _atomic = require("../actions/atomic");

var _condtions = require("../actions/condtions");

var _leafReducerArray = require("./array/leafReducerArray");

var _leafReducerObject = require("./object/leafReducerObject");

var _leafReducerString = require("./string/leafReducerString");

var _leafReducerBoolean = require("./boolean/leafReducerBoolean");

var _leafReducerNumber = require("./number/leafReducerNumber");

var _leafReducerCustom = require("./custom/leafReducerCustom");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj["default"] = obj; return newObj; } }

var leafReducer = function leafReducer(leafState, action, wholeState, initialWhole, reducersDict) {
  var _action$leaf = action.leaf,
      leaf = _action$leaf === void 0 ? {} : _action$leaf,
      payload = action.payload;
  var condition = leaf.condition,
      custom = leaf.custom,
      creatorKey = leaf.creatorKey,
      path = leaf.path;
  return (0, _immer["default"])(leafState, function (draftLeafState) {
    // Custom actions
    if (custom) {
      return (0, _leafReducerCustom.leafReducerCustom)(reducersDict, draftLeafState, action, wholeState);
    } // Type-specific actions


    switch (condition) {
      case _condtions.conditions.ARRAY:
        return (0, _leafReducerArray.leafReducerArray)(draftLeafState, {
          creatorKey: creatorKey,
          payload: payload
        });

      case _condtions.conditions.BOOLEAN:
        return (0, _leafReducerBoolean.leafReducerBoolean)(draftLeafState, {
          creatorKey: creatorKey
        });

      case _condtions.conditions.NUMBER:
        return (0, _leafReducerNumber.leafReducerNumber)(draftLeafState, {
          creatorKey: creatorKey,
          payload: payload
        });

      case _condtions.conditions.OBJECT:
        return (0, _leafReducerObject.leafReducerObject)(draftLeafState, {
          creatorKey: creatorKey,
          payload: payload
        });

      case _condtions.conditions.STRING:
        return (0, _leafReducerString.leafReducerString)(draftLeafState, {
          creatorKey: creatorKey,
          payload: payload
        });
    } // Type-agnostic actions


    switch (creatorKey) {
      case _atomic.atomicActions.APPLY:
        return apply(payload, draftLeafState, wholeState);

      case _atomic.atomicActions.CLEAR:
        return clear(draftLeafState, payload);

      case _atomic.atomicActions.RESET:
        return reset(initialWhole, path);

      case _atomic.atomicActions.UPDATE:
        return payload;

      default:
        return draftLeafState;
    }
  });
};

exports.leafReducer = leafReducer;

var apply = function apply(callback, leafState, wholeState) {
  return callback(leafState, wholeState);
};

var clear = function clear(leafState, toNull) {
  if (toNull) {
    return null;
  } else if (RA.isBoolean(leafState)) {
    return false;
  } else if (RA.isString(leafState)) {
    return '';
  } else if (RA.isArray(leafState)) {
    return [];
  } else if (RA.isNumber(leafState)) {
    return 0;
  } else if (RA.isPlainObject(leafState)) {
    return {};
  }
};

var reset = function reset(initialWholeState, path) {
  return path.length >= 1 ? R.path(path, initialWholeState) : initialWholeState;
};